unit Data;

// Copyright  1997 by Jon Shemitz, all rights reserved.
// Permission is hereby granted to freely use, modify, and
// distribute this source code PROVIDED that all six lines of
// this copyright and contact notice are included without any
// changes. Questions? Comments? Offers of work?
// mailto:jon@midnightbeach.com

// ------------------------------------------------------------

// Data modules typically contain the code to read/write model
// objects to a database. This one just contains some dummy
// routines that generate canned data.

interface

uses
  Classes, Forms, Models;

type
  TPersonID = integer;
  TPerson = class (TModel)
            ID:        TPersonID;
            Name:      string;
            BirthDate: TDateTime;
            TaxId,
            Address1,
            Address2,
            Address3,
            Phone1,
            Phone2:    string;

            procedure Reset; // flag as "unset"
            end;

  TEmployeeID = integer;
  TEmployee   = class (TModel)
                ID:         TEmployeeID;
                Employee,
                Supervisor: TPerson;
                HireDate:   TDateTime;

                constructor Create;
                destructor Destroy; override;
                procedure  Reset; // flag as "unset"
                end;

type
  TDataModel = class(TDataModule)
    procedure DataModelCreate(Sender: TObject);
    procedure DataModelDestroy(Sender: TObject);
  private
    { Private declarations }
    fSupervisors: TList;

    function GetSupervisorCount: integer;
    function GetSupervisor(N: integer): TPersonID;
  public
    { Public declarations }

    // TPerson create/load/save
    function NewPerson: TPerson;
    function LoadPerson(ID: TPersonID): TPerson;
    procedure GetPerson(Person: TPerson; ID: TPersonID);
    procedure SavePerson(Person: TPerson);

    // TEmployee create/load/save
    function NewEmployee: TEmployee;
    function LoadEmployee(ID: TEmployeeID): TEmployee;
    procedure GetEmployee(Employee: TEmployee; ID: TEmployeeID);
    procedure SaveEmployee(Person: TEmployee);

    // Supervisor list
    property SupervisorCount: integer read GetSupervisorCount;
    property Supervisor[N: integer]: TPersonID read GetSupervisor;
    function IndexOfSupervisor(ID: TPersonID): integer;

    // misc
    function GetPersonName(ID: TPersonID): string;
    function GetEmployeeName(ID: TEmployeeID): string;
  end;

var
  DataModel: TDataModel;

implementation

uses SysUtils;

{$R *.DFM}

procedure TDataModel.DataModelCreate(Sender: TObject);
begin
  fSupervisors := TList.Create;
  fSupervisors.Add(pointer(1)); // dummy data
  fSupervisors.Add(pointer(2));
end; // TDataModel.DataModelCreate

procedure TDataModel.DataModelDestroy(Sender: TObject);
begin
  fSupervisors.Free;
end; // TDataModel.DataModelDestroy

// Supervisor list

function TDataModel.GetSupervisorCount: integer;
begin
  Result := fSupervisors.Count;
end; // TDataModel.GetSupervisorCount

function TDataModel.GetSupervisor(N: integer): TPersonID;
begin
  Result := TPersonID( fSupervisors[N] );
end; // TDataModel.GetSupervisor

function TDataModel.IndexOfSupervisor(ID: TPersonID): integer;
begin
  Result := fSupervisors.IndexOf( pointer(ID) );
end; // TDataModel.IndexOfSupervisor

// TPerson create/load/save

procedure TPerson.Reset;
begin
  ID        := -1;
  Name      := '';
  BirthDate := 0;
  TaxId     := '';
  Address1  := '';
  Address2  := '';
  Address3  := '';
  Phone1    := '';
  Phone2    := '';
end; // TDataModel.Reset

function TDataModel.NewPerson: TPerson;
begin
  Result := TPerson.Create;
  Result.Reset;
end; // TDataModel.NewPerson

function TDataModel.LoadPerson(ID: TPersonID): TPerson;
begin
  Result := TPerson.Create;
  GetPerson(Result, ID);
end; // TDataModel.LoadPerson

type
  Ids = 1..3;

procedure TDataModel.GetPerson(Person: TPerson; ID: TPersonID);
const
  Name:      array[Ids] of string    =
             ('John Jacob Jingleheimer Schmidt',
              'Revd. Wicks Cherrycoke',
              'Luke Skywalker');
  Birthdate: array[Ids] of string =
             ('12/26/58', '1/9/61', '9/26/88');
  TaxId:     array[Ids] of string =
             ('123-45-6789', '987-65-4321', '555-55-5555');
  Address1:  array[Ids] of string =
             ('11 Jingle Lane',
              '121 Mason Place',
              '12321 Right Way');
  Address2:  array[Ids] of string =
             ('Becket, Massachusetts 01234',
              'Philadelphia, Pennsylvania 12345',
              'Sausalito, California 98765');
  Phone1:    array[Ids] of string =
             ('123-555-1212', '456-555-1212', '789-555-1212');
begin
  if (ID < Low(Ids)) or (ID > High(Ids))
    then Person.Reset
    else begin
         Person.ID        := ID;
         Person.Name      := Name[ID];
         Person.BirthDate := StrToDate(Birthdate[Id]);
         Person.TaxId     := TaxId[Id];
         Person.Address1  := Address1[Id];
         Person.Address2  := Address2[Id];
         Person.Address3  := '';
         Person.Phone1    := Phone1[Id];
         Person.Phone2    := '';
         end;
end; // TDataModel.GetPerson

procedure TDataModel.SavePerson(Person: TPerson);
begin
end; // TDataModel.SavePerson

// TEmployee create/load/save

constructor TEmployee.Create;
begin
  inherited;
  Employee   := TPerson.Create; Employee.Reset;
  Supervisor := TPerson.Create; Supervisor.Reset;
end; // TEmployee.Create

destructor TEmployee.Destroy;
begin
  Employee.Free;
  Supervisor.Free;
end; // TEmployee.Destroy

procedure TEmployee.Reset;
begin
  ID       := -1;
  Employee.Reset;
  Supervisor.Reset;
  HireDate := 0;
end; // TEmployee.Reset

function TDataModel.NewEmployee: TEmployee;
begin
  Result := TEmployee.Create;
  Result.Reset;
end; // TDataModel.NewEmployee

function TDataModel.LoadEmployee(ID: TEmployeeID): TEmployee;
begin
  Result := TEmployee.Create;
  GetEmployee(Result, ID);
end; // TDataModel.LoadEmployee

procedure TDataModel.GetEmployee(Employee: TEmployee; ID: TEmployeeID);
const
  Supervisor: array[Ids] of TEmployeeID = (-1, -1, 2);
  Hiredate:   array[Ids] of string =
              ('1/2/93', '3/4/95', '5/6/97');
begin
  if (ID < Low(Ids)) or (ID > High(Ids))
    then Employee.Reset
    else begin
         Employee.ID       := ID;
         GetPerson(Employee.Employee,   ID);
         GetPerson(Employee.Supervisor, Supervisor[Id]);
         Employee.HireDate := StrToDate(HireDate[Id]);
         end;
end; // TDataModel.GetEmployee

procedure TDataModel.SaveEmployee(Person: TEmployee);
begin
end; // TDataModel.SaveEmployee

function TDataModel.GetPersonName(ID: TPersonID): string;
var
  Person: TPerson;
begin
  Person := LoadPerson(ID);
  try
    Result := Person.Name;
  finally Person.Free; end;
end; // TDataModel.GetPersonName

function TDataModel.GetEmployeeName(ID: TEmployeeID): string;
begin
  Result := GetPersonName(ID);
end; // TDataModel.GetEmployeeName

end.

