unit PropertySheets;

// Copyright  1997 by Jon Shemitz, all rights reserved.
// Permission is hereby granted to freely use, modify, and
// distribute this source code PROVIDED that all six lines of
// this copyright and contact notice are included without any
// changes. Questions? Comments? Offers of work?
// mailto:jon@midnightbeach.com

// ------------------------------------------------------------

// An AbstractPropertySheet is a superclass for an actual
// property sheet that supplies Caption, and an Initialize
// procedure that specifies tab captions and view classes.

interface

{ $define Test}	// Disables "StartedInvalid" code

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, ComCtrls, StdCtrls, Buttons, ExtCtrls,
  Models, Views, Embedded, ModelEditors;

type
  TAbstractPropertySheet = class(TModelEditor, IFrame)
    ButtonPnl: TPanel;
    RightPnl: TPanel;
    OkBtn: TBitBtn;
    CancelBtn: TBitBtn;
    PageControl: TPageControl;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    fViews: TList;
    InvalidCount:   integer;
    StartedInvalid: boolean;

    procedure OnValidChanged( ChangingObject: TObject;
                              View:           IView);
    function GetValid: boolean;
  protected
    procedure SetReadOnly(Value: boolean);       override;

    procedure InitializeSheet( Captions: array of string;
                               Views:    array of TViewClass );
    function  RunEditor(Model: TModel): boolean; override;
    procedure Finalize;                          override;
  public
    property Valid: boolean read GetValid;
  end;

implementation

{$R *.DFM}

procedure TAbstractPropertySheet.FormCreate(Sender: TObject);
begin
  fViews := TList.Create;
end; // TAbstractPropertySheet.FormCreate

procedure TAbstractPropertySheet.FormDestroy(Sender: TObject);
begin
  fViews.Free;
end; // TAbstractPropertySheet.FormDestroy

procedure TAbstractPropertySheet.InitializeSheet(
  Captions:   array of string;
  Views:      array of TViewClass );
var
  MaxSpan:    TSpan;
  Index:      integer;
  Sheet:      TTabSheet;
  ActualView: TAbstractView;
begin
  Assert( fViews.Count = 0,
          'Should only call ' + Name + '.InitializeSheet once' );
  Assert( High(Captions) >= Low(Captions), // can use Slice() to
          'Must have at least one tab' );  // pass empty arrays
  Assert( High(Captions) = High(Views),
          'Must have same number of Captions as of Views' );

  MaxSpan := Point(0, 0);

  for Index := Low(Captions) to High(Captions) do
    begin
    Sheet := TTabSheet.Create(Self);
    with Sheet do
      begin
      PageControl := Self.PageControl;
      Caption     := Captions[Index];
      end; // with Sheet
    ActualView := Views[Index].CreateEmbedded( Self,
                                               Sheet,
                                               efmTopLeft );
    fViews.Add(ActualView);
    ActualView.AddNotifiee(Self);
    MaxSpan := UnionSpan(MaxSpan, ActualView.Span);
    end; // for

  Sheet  := PageControl.ActivePage;
  Width  := (Width  - Sheet.Width)  + MaxSpan.X;
  Height := (Height - Sheet.Height) + MaxSpan.Y;
end; // TAbstractPropertySheet.InitializeSheet

procedure TAbstractPropertySheet.SetReadOnly(Value: boolean);
var
  Index: integer;
begin
  inherited;
  for Index := 0 to fViews.Count - 1 do
    begin
    Assert(TObject(fViews[Index]) is TAbstractView);
    TAbstractView(fViews[Index]).ReadOnly := Value;
    end; // for
end; // TAbstractPropertySheet.SetReadOnly

function TAbstractPropertySheet.RunEditor(
  Model: TModel): boolean;
var
  Index: integer;
begin
  // Show existing data
  for Index := 0 to fViews.Count - 1 do
    begin
    Assert(TObject(fViews[Index]) is TAbstractView);
    TAbstractView(fViews[Index]).ReadFromModel(Model);
    end;

  // Check its validity
  InvalidCount := 0;
  for Index := 0 to fViews.Count - 1 do
    if not TAbstractView(fViews[Index]).Valid then
      Inc(InvalidCount);
  StartedInvalid := InvalidCount <> 0;
  OkBtn.Enabled := (InvalidCount = 0)
                   {$ifNdef Test} or StartedInvalid {$endif} ;

  // Run the dialog
  Result := ShowModal = mrOK;

  // Read any changes
  if Result then
    for Index := 0 to fViews.Count - 1 do
      TAbstractView(fViews[Index]).WriteToModel(Model);
end; // TAbstractPropertySheet.RunSheet

procedure TAbstractPropertySheet.Finalize;
var
  Index: integer;
begin
  for Index := 0 to fViews.Count - 1 do
    begin
    Assert(TObject(fViews[Index]) is TAbstractView);
    TAbstractView(fViews[Index]).RemoveNotifiee(Self);
    // Release existing interface reference
    end;
end; // TAbstractPropertySheet.FinalizeSheet

procedure TAbstractPropertySheet.OnValidChanged(
  ChangingObject: TObject;
  View:           IView );
begin
  if View.Valid
    then Dec(InvalidCount)
    else Inc(InvalidCount);
  OkBtn.Enabled := (InvalidCount = 0)
                   {$ifNdef Test} or StartedInvalid {$endif} ;
end; // TAbstractPropertySheet.OnValidChanged

function TAbstractPropertySheet.GetValid: boolean;
begin
  Result := InvalidCount = 0;
end; // TAbstractPropertySheet.GetValid

end.

