unit Wizards; {$T+,J+}

// Copyright  1997 by Jon Shemitz, all rights reserved.
// Permission is hereby granted to freely use, modify, and
// distribute this source code PROVIDED that all six lines of
// this copyright and contact notice are included without any
// changes. Questions? Comments? Offers of work?
// mailto:jon@midnightbeach.com

// ------------------------------------------------------------

// An AbstractWizard is a superclass for an actual Wizard that
// supplies a WizardImage, a Caption, and an Initialize
// procedure that specifies page captions and view classes.

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, ComCtrls, ExtCtrls, StdCtrls, Buttons,
  Models, Views, Embedded, ModelEditors;

type
  TAbstractWizard = class(TModelEditor, IFrame)
    PicturePnl: TPanel;
    FramePnl: TPanel;
    ButtonPnl: TPanel;
    FloatingPnl: TPanel;
    PrevBtn: TBitBtn;
    NextBtn: TBitBtn;
    OkBtn: TBitBtn;
    CancelBtn: TBitBtn;
    Bevel: TBevel;
    WizardImage: TImage;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PrevBtnClick(Sender: TObject);
    procedure NextBtnClick(Sender: TObject);
  private
    fCaptionHeight: integer;
    fPageCount:     integer;
    fCurrentPage:   integer;
    fPanels:        TList;
    fViews:         TList;

    function CreatePage( Caption: string;
                         View: TViewClass ): TPoint;
    procedure SetCurrentPage(Value: integer);
  protected
    property PageCount:   integer read  fPageCount;
    property CurrentPage: integer read  fCurrentPage
                                  write SetCurrentPage;

    procedure OnValidChanged(ChangingObject: TObject;
                             View:           IView);
    procedure InitializeWizard( Captions: array of string;
                                Views:    array of TViewClass );
    function  RunEditor(Model: TModel): boolean; override;
    procedure Finalize;                          override;

    function  GetReadOnly: boolean;              override;
    procedure SetReadOnly(Value: boolean);       override;
  public
    property CaptionHeight: integer read  fCaptionHeight
                                    write fCaptionHeight;
    // CaptionHeight, in parts per thousand of the height of a
    // single line of text (ie, 2250 is two and a quarter lines).
    // Must be set *before* calling Initialize/EditModel().
  end;

implementation

{$R *.DFM}

procedure TAbstractWizard.FormCreate(Sender: TObject);
begin
  fPanels       := TList.Create;
  fViews        := TList.Create;
  fPageCount    := 0;
  CaptionHeight := 2250; // default
end; // TAbstractWizard.FormCreate

procedure TAbstractWizard.FormDestroy(Sender: TObject);
begin
  fPanels.Free; // Freeing the Wizard frees the panels ....
  fViews.Free;
end; // TAbstractWizard.FormDestroy

function  TAbstractWizard.GetReadOnly: boolean;
begin
  Result := False; // Wizards are never read-only
end; // TAbstractWizard.GetReadOnly

procedure TAbstractWizard.SetReadOnly(Value: boolean);
begin
  // Wizards are never read-only
end; // TAbstractWizard.SetReadOnly

procedure TAbstractWizard.SetCurrentPage(Value: integer);
var
  LastPage, PageIsValid: boolean;
begin
  Assert(TObject(fPanels[Value]) is TPanel);
  Assert(TObject(fViews[Value]) is TAbstractView);
  // Using Assert(is) with 'blind' casts gives us the debugging
  // safety of "as" without the (modest) performance hit

  fCurrentPage := Value;
  TPanel(fPanels[Value]).BringToFront;

  LastPage := Value = fPageCount;
  PageIsValid := TAbstractView(fViews[Value]).Valid;

  PrevBtn.Enabled := Value > 0;
  NextBtn.Enabled := PageIsValid and (not LastPage);
  OkBtn.Enabled   := PageIsValid and LastPage;
end; // TAbstractWizard.SetCurrentPage

procedure TAbstractWizard.OnValidChanged(
  ChangingObject: TObject;
  View:           IView );
begin
  if ChangingObject = fViews[CurrentPage] then
    CurrentPage := CurrentPage; // reset Next/OK enabled state
end; // TAbstractWizard.OnValidChanged

procedure TAbstractWizard.PrevBtnClick(Sender: TObject);
begin
  CurrentPage := CurrentPage - 1;
end; // TAbstractWizard.PrevBtnClick

procedure TAbstractWizard.NextBtnClick(Sender: TObject);
begin
  CurrentPage := CurrentPage + 1;
end; // TAbstractWizard.NextBtnClick

  function NewPanel( _Owner:  TComponent;
                     _Parent: TWinControl;
                     _Align:  TAlign): TPanel;
  begin
    Result := TPanel.Create(_Owner);
    with Result do
      begin
      Parent     := _Parent;
      Align      := _Align;
      BevelInner := bvNone;
      BevelOuter := bvNone;
      end; // with Result
  end; // NewPanel

  function AllChars: string;
  var
    Index: integer;
  const
    Cache: string = '';
  begin
    if Cache = '' then
      begin // first call
      SetLength(Cache, 256);
      for Index := 1 to 256 do Cache[Index] := char(Index - 1);
      end;  // first call
    Result := Cache;
  end;

function TAbstractWizard.CreatePage( Caption: string;
                                     View: TViewClass): TPoint;

var
  MainPnl, CaptionPnl, ViewPnl: TPanel;
  CaptionLbl:                   TLabel;
  ActualView:                   TAbstractView;
begin
  // Create the three panels
  MainPnl    := NewPanel(Self, FramePnl, alClient);
  CaptionPnl := NewPanel(Self, MainPnl,  alTop);
  ViewPnl    := NewPanel(Self, MainPnl,  alClient);
  fPanels.Add(MainPnl);
  // Fill in the caption
  CaptionPnl.BorderWidth := 2;
  CaptionLbl             := TLabel.Create(Self);
  CaptionLbl.Parent      := CaptionPnl;
  CaptionLbl.Align       := alClient;
  CaptionLbl.Alignment   := taLeftJustify;
  CaptionLbl.Wordwrap    := True;
  CaptionLbl.Caption     := Caption;

  CaptionPnl.Height := (CaptionLbl.Canvas.TextHeight(AllChars)
                        * CaptionHeight) div 1000;
  // Create the view
  ActualView := View.CreateEmbedded(Self, ViewPnl, efmTopLeft);
  fViews.Add(ActualView);
  ActualView.AddNotifiee(Self);

  Result := ActualView.SpanEx(0, CaptionPnl.Height);
end; // TAbstractWizard.CreatePage

procedure TAbstractWizard.InitializeWizard(
  Captions:   array of string;
  Views:      array of TViewClass );
var
  Index:   integer;
  MaxSpan: TSpan;
begin
  Assert( PageCount = 0, 'Can only call Initialize() once' );
  Assert( High(Captions) >= Low(Captions), // can use Slice() to
          'Must have at least one page' ); // pass empty arrays
  Assert( High(Captions) = High(Views),
          'Must have same number of Captions as of Views' );

  MaxSpan := Point(0, 0);
  for Index := Low(Captions) to High(Captions) do
    MaxSpan :=
      UnionSpan( MaxSpan,
                 CreatePage(Captions[Index], Views[Index]) );

  Width  := (Width  - FramePnl.Width)  + MaxSpan.X;
  Height := (Height - FramePnl.Height) + MaxSpan.Y;

  fPageCount := High(Captions) - Low(Captions);
  CurrentPage := 0;
end; // TAbstractWizard.InitializeWizard

function  TAbstractWizard.RunEditor(Model: TModel): boolean;
var
  Index: integer;
begin
  // Show existing (default) data
  for Index := 0 to fViews.Count - 1 do
    begin
    Assert(TObject(fViews[Index]) is TAbstractView);
    TAbstractView(fViews[Index]).ReadFromModel(Model);
    end;

  // Execute
  Result := ShowModal = mrOK;

  // Read user data
  if Result then
    for Index := 0 to fViews.Count - 1 do 
      TAbstractView(fViews[Index]).WriteToModel(Model);
end; // TAbstractWizard.RunEditor

procedure TAbstractWizard.Finalize;
var
  Index: integer;
begin
  // cleanup
  for Index := 0 to fViews.Count - 1 do 
    (TObject(fViews[Index]) as TAbstractView).RemoveNotifiee(Self);
    // Release existing interface reference
end; // TAbstractWizard.Finalize

end.

